package com.grack.prologmachine;

import java.util.*;

/**
 * A class which contains the Most General Unifier (MGU) set: the set which 
 * unifies two terms.
 * @author Matthew Mastracci
 */
public class MostGeneralUnifier {
    /** Internal list of substitutions */
    private HashMap _substitutions;

    /**
     * Creates an empty MGU.
     */
    public MostGeneralUnifier() {
        _substitutions = new HashMap();
    }

    /**
     * Adds a new substitution to the MGU, as well as updating the old 
     * substitutions with the new variable.
     * @param v The variable to replace.
     * @param u The value to replace the variable with.
     */
    public void addSubstitution( Variable v, Unifiable u ) {
        _substitutions.put( v, u );
        Set s = _substitutions.entrySet();
        Iterator i = s.iterator();
        while ( i.hasNext() ) {
            Unifiable un = (Unifiable)(((Map.Entry)i.next()).getValue());
            if ( un instanceof Predicate ) {
                applySubstitution( (Predicate)un );
            }
        }
    }

    /**
     * Applies the current substitution set to the given Predicate.
     * @param p The Predicate to substiture.
     */
    public void applySubstitution( Predicate p ) {
        for ( int i = 0; i < p.argumentCount(); i++ ) {
            Unifiable u = p.getArgument( i );
            if ( u instanceof Predicate )
                applySubstitution( (Predicate)u );
            if ( u instanceof Variable )
                if ( _substitutions.containsKey( (Variable)u ) ) {
                    p.setArgument( i, (Unifiable)_substitutions.get( (Variable)u ) );
                }
        }
    }

    /**
     * Converts the MGU to a string represenation.
     * @return The string representation.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append( "{ " );
        Set s = _substitutions.keySet();
        Iterator i = s.iterator();
        boolean isFirst = true;
        while (i.hasNext()) {      
            if ( isFirst ) {
                isFirst = false;
            } else {
                sb.append( ", " );
            }
            Variable v = (Variable)i.next();
            sb.append( v + " = " + _substitutions.get( v ) );
        }
        if ( _substitutions.size() > 0 )
            sb.append( " " );
        sb.append( "}" );

        return sb.toString();
    }
}
