package com.grack.prologmachine;

/**
 * Holds a signature for a predicate - its name and arity.
 * @author Matthew Mastracci
 */
public class PredicateSignature {
    private String _name;
    private int _arity;

    /**
     * Constructs a new predicate signature.
     * @param name The name of the predicate (ie: "animal" for animal/1)
     * @param name The arity of the predicate (ie: 1 for animal/1)
     */
    public PredicateSignature( String name, int arity ) {
        _name = name;
        _arity = arity;
    }

    /**
     * Converts the PredicateSignature to a String.
     * @return The string representation (name/arity).
     */
    public String toString() {
        return _name + "/" + _arity;
    }
    
    /**
     * Returns the name portion of the signature.
     * @return The name portion of the signature.
     */
    public String getName() {
        return _name;
    }

    /**
     * Returns the arity portion of the signature.
     * @return The arity portion of the signature.
     */
    public int getArity() {
        return _arity;
    }

    /**
     * Returns the hashcode of the signature.
     * @return The hashcode of the signature.
     */
    public int hashCode() {
        return _name.hashCode() + _arity;
    }

    /**
     * Tests to see if two signatures are equal.
     * @param o The other object.
     * @return true if they have the same name and arity.
     */
    public boolean equals( Object o ) {
        PredicateSignature ps = (PredicateSignature)o;
        return ( _name.equals( ps._name ) && _arity == ps._arity );
    }
}
