package com.grack.prologmachine.test;

import junit.framework.*;
import com.grack.prologmachine.*;

public class TestMostGeneralUnifier extends TestCase {
    public TestMostGeneralUnifier( String name ) {
        super( name );
    }

    public static Test suite() {
        return new TestSuite( TestMostGeneralUnifier.class );
    }

    public void setUp() {
    }

    public void tearDown() {
    }

    public void testEmpty() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();

        assert( mgu.toString().equals( "{ }" ) );
    }

    public void testSimple() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        mgu.addSubstitution( new Variable( "X" ), new Predicate( "cow", 1, new Variable( "Y" ) ) );
        assert( mgu.toString().equals( "{ X = cow(Y) }" ) );
    }

    public void testBig1() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        mgu.addSubstitution( new Variable( "Y" ), new Predicate( "cow", 1, new Variable( "W" ) ) );
        mgu.addSubstitution( new Variable( "X" ), new Predicate( "cow", 1, new Variable( "Y" ) ) );
        mgu.addSubstitution( new Variable( "Z" ), new Predicate( "cow", 1, new Variable( "X" ) ) );
        Predicate p;

        p = new Predicate( "foo", 1, new Variable( "Y" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(W))" ) );

        p = new Predicate( "foo", 1, new Variable( "X" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(cow(W)))" ) );

        p = new Predicate( "foo", 1, new Variable( "Z" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(cow(cow(W))))" ) );

        Unifiable[] arguments = { new Variable( "X" ), new Variable( "Y" ), new Variable( "Z" ), new Variable( "W" ) };
        p = new Predicate( "foo", 4, arguments );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(cow(W)),cow(W),cow(cow(cow(W))),W)" ) );
    }

    public void testBig2() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        mgu.addSubstitution( new Variable( "Z" ), new Predicate( "cow", 1, new Variable( "X" ) ) );
        mgu.addSubstitution( new Variable( "X" ), new Predicate( "cow", 1, new Variable( "Y" ) ) );
        mgu.addSubstitution( new Variable( "Y" ), new Predicate( "cow", 1, new Variable( "W" ) ) );
        Predicate p;

        p = new Predicate( "foo", 1, new Variable( "Y" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(W))" ) );

        p = new Predicate( "foo", 1, new Variable( "X" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(cow(W)))" ) );

        p = new Predicate( "foo", 1, new Variable( "Z" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(cow(cow(W))))" ) );

        Unifiable[] arguments = { new Variable( "X" ), new Variable( "Y" ), new Variable( "Z" ), new Variable( "W" ) };
        p = new Predicate( "foo", 4, arguments );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "foo(cow(cow(W)),cow(W),cow(cow(cow(W))),W)" ) );
    }
}






