package com.grack.prologmachine.test;

import junit.framework.*;
import com.grack.prologmachine.*;

public class TestUnifiable extends TestCase {
    public TestUnifiable( String name ) {
        super( name );
    }

    public static Test suite() {
        return new TestSuite( TestUnifiable.class );
    }

    public void setUp() {
    }

    public void tearDown() {
    }
    
    public void testNoArguments() {
        Predicate pred = new Predicate( "foo", 0 );
        assert( pred.toString().equals( "foo" ) );
        assert( pred.isValid() );
    }

    public void testWildcard() {
        Predicate pred = new Predicate( "foo", 1, new Wildcard() );
        assert( pred.toString().equals( "foo(_)" ) );
        assert( pred.isValid() );
    }

    public void testEmptyList() {
        Predicate pred = new Predicate( "foo", 1, new EmptyList() );
        assert( pred.toString().equals( "foo([])" ) );
        assert( pred.isValid() );
    }

    public void testIntegerNumber() {
        Predicate pred = new Predicate( "foo", 1, new IntegerNumber( 4 ) );
        assert( pred.toString().equals( "foo(4)" ) );
        assert( pred.isValid() );
    }

    public void testFloatNumber() {
        Predicate pred = new Predicate( "foo", 1, new FloatNumber( 4.5 ) );
        assert( pred.toString().equals( "foo(4.5)" ) );
        assert( pred.isValid() );
    }

    public void testSimpleCompoundPredicate() {
        Predicate pred = new Predicate( "foo", 1, new Predicate( "moo", 1, new EmptyList() ) );
        assert( pred.toString().equals( "foo(moo([]))" ) );
        assert( pred.isValid() );       
    }

    public void testComplexCompoundPredicate() {
        Predicate pred = new Predicate( "foo", 3 );
        pred.addArgument( new Predicate( "moo", 1, new EmptyList() ) );
        pred.addArgument( new Predicate( "moo", 1, new Variable( "X" ) ) );
        Unifiable[] arguments = { new Variable( "X" ), new Variable( "Y" ) };
        pred.addArgument( new Predicate( "moo", 2, arguments ) );
        assert( pred.toString().equals( "foo(moo([]),moo(X),moo(X,Y))" ) );
        assert( pred.isValid() );               
    }

    public void testArity() {
        Predicate pred = new Predicate( "foo", 3 );
        pred.addArgument( new Predicate( "moo", 1, new EmptyList() ) );
        pred.addArgument( new Predicate( "moo", 1, new Variable( "X" ) ) );
        Unifiable[] arguments = { new Variable( "X" ), new Variable( "Y" ) };
        pred.addArgument( new Predicate( "moo", 2, arguments ) );
        assert( pred.getArity() == 3 );
        assert( pred.isValid() );                       
    }

    public void testName() {
        Predicate pred = new Predicate( "foo", 3 );
        pred.addArgument( new Predicate( "moo", 1, new EmptyList() ) );
        pred.addArgument( new Predicate( "moo", 1, new Variable( "X" ) ) );
        Unifiable[] arguments = { new Variable( "X" ), new Variable( "Y" ) };
        pred.addArgument( new Predicate( "moo", 2, arguments ) );
        assert( pred.getName() == "foo" );
        assert( pred.isValid() );                       
    }

    public void testSignature() {
        Predicate pred = new Predicate( "foo", 3 );
        pred.addArgument( new Predicate( "moo", 1, new EmptyList() ) );
        pred.addArgument( new Predicate( "moo", 1, new Variable( "X" ) ) );
        Unifiable[] arguments = { new Variable( "X" ), new Variable( "Y" ) };
        pred.addArgument( new Predicate( "moo", 2, arguments ) );

        PredicateSignature ps = pred.getSignature();
        assert( ps.getName() == "foo" );
        assert( ps.getArity() == 3 );
        assert( pred.isValid() );                       
    }
}






