package com.grack.prologmachine.test;

import junit.framework.*;
import com.grack.prologmachine.*;

public class TestUnify extends TestCase {
    public TestUnify( String name ) {
        super( name );
    }

    public static Test suite() {
        return new TestSuite( TestUnify.class );
    }

    public void setUp() {
    }

    public void tearDown() {
    }

    public void testSimple1() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        assert( Term.unify( new Variable( "X" ), new Predicate( "cow", 1, new Variable( "Y" ) ), mgu ) );
        assert( mgu.toString().equals( "{ X = cow(Y) }" ) );
    }

    public void testSimple2() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        assert( Term.unify( new Predicate( "cow", 1, new Variable( "Y" ) ), new Variable( "X" ), mgu ) );
        assert( mgu.toString().equals( "{ X = cow(Y) }" ) );
    }

    public void testSimple3() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        assert( Term.unify( new Variable( "Y" ), new Variable( "X" ), mgu ) );
        assert( mgu.toString().equals( "{ Y = X }" ) );        
    }

    public void testSimple4() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        assert( Term.unify( new Predicate( "cow", 1, new Variable( "Y" ) ), new Predicate( "cow", 1, new Variable( "X" ) ), mgu ) );
        assert( mgu.toString().equals( "{ Y = X }" ) );        
    }

    public void testSimple5() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        Unifiable[] arg1 = { new Variable( "X" ), new Variable( "Y" ) };
        Unifiable[] arg2 = { new IntegerNumber( 1 ), new Variable( "X" ) };
        assert( Term.unify( new Predicate( "cow", 2, arg1 ), 
                            new Predicate( "cow", 2, arg2 ),
                            mgu ) );
        Predicate p;
        p = new Predicate( "cow", 1, new Variable( "X" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "cow(1)" ) );
        p = new Predicate( "cow", 1, new Variable( "Y" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "cow(1)" ) );
    }

    public void testSimple6() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        assert( Term.unify( new IntegerNumber( 1 ), 
                            new IntegerNumber( 1 ),
                            mgu ) );
        assert( mgu.toString().equals( "{ }" ) );        
    }

    public void testSimple7() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        assert( !Term.unify( new IntegerNumber( 1 ), 
                            new IntegerNumber( 2 ),
                            mgu ) );
    }

    public void testSimple8() {
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        Unifiable[] arg1 = { new Variable( "X" ), new Variable( "Y" ) };
        Unifiable[] arg2 = { new Predicate( "g", 1, new Variable( "Y" ) ), new Predicate( "a", 0 ) };
        assert( Term.unify( new Predicate( "f", 2, arg1 ),
                            new Predicate( "f", 2, arg2 ),
                            mgu ) );
        
        Predicate p;
        p = new Predicate( "cow", 1, new Variable( "X" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "cow(g(a))" ) );
        p = new Predicate( "cow", 1, new Variable( "Y" ) );
        mgu.applySubstitution( p );
        assert( p.toString().equals( "cow(a)" ) );
        
    }
    
    public void testSimple9() {        
        MostGeneralUnifier mgu = new MostGeneralUnifier();
        Unifiable[] arg1 = { new Variable( "X" ), new Variable( "X" ), new Variable( "X" ) };
        Unifiable[] arg2 = { new Variable( "Y" ), new Predicate( "g", 1, new Variable( "Y" ) ), new Predicate( "a", 0 ) };
        // need to implement unify with occurs check!
        /*
          assert( !Term.unifyWithOccursCheck( new Predicate( "f", 3, arg1 ),
                             new Predicate( "f", 3, arg2 ),
                             mgu ) );                
        */
    }
}






